package examples.xml.bridge;

/*------------------------------------------------------------------------------*/
/*                    MQSeries Everyplace for Multiplatforms                    */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5765-E63                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 1999                                                 */
/* -----------------------------------------------------------------------------*/
import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.*;
import com.ibm.mq.*;
import java.util.*;
import com.ibm.mqe.xml.*;

/*****************************************************************************
 *
 * A XML MQ to MQe transformer class.
 *
 *****************************************************************************/

public class MQeXMLTransformer implements MQeTransformerInterface {



	/**
	 * An MQe message transformed to XML inside a MQ message
	 */
	public static final int MQMT_XML				= 114;

	//the next member is the MQeFields-XML transformer we will use.
	private MQeXMLConverter converter = null;

	/**
	 * Tells the transformer of any parameters that were supplied
	 * with the transformer definition. This transformer does not use
	 * any of the supplied parameters.
	 * @param StringTokenizer containing the transformer
	 * definition parameters.
	 * @exception java.lang.Exception If the parameters are invalid or
	 * the transformer cannot be initialised.
	 */
	public void activate(StringTokenizer params) throws Exception 
	{
		ErrorHandler error_handler = new DefaultErrorHandler();
		converter = new MQeXMLConverter(error_handler);
		converter.setDefault_Class("com.ibm.mqe.MQeMsgObject");
	}
	/***************************************************************************
	 * Convert a MQ format message into a MQe format message. This method is
	 * called by the MQ/MQe bridge.
	 *
	 * @param msg            The MQSeries message object we are to transform.
	 * @param remoteQMgrName The Destination Remote Queue Manager Name
	 *                       (from the MQ Remote Queue Definition)
	 * @param remoteQName    The Destination Remote Queue Name (from the
	 *                       MQ Remote Queue Definition).
	 * @return               The equivalent MQe format message.
	 * @exception java.lang.Exception  propagated from the called methods.
	 ***************************************************************************/
	public MQeMsgObject transform(MQMessage msg,
								  String remoteQMgrName,
								  String remoteQName) throws Exception
	{
		
		// We get the data of the MQMessage, this must be a XML String
		byte[] userData = new byte[msg.getDataLength()];
		msg.readFully(userData);
		
		// We inflate the MQeMessageObject with these data
		MQeMsgObject result = (MQeMsgObject)converter.toMQeFields(userData);

		// We set the various parameters of the MQMessage into the MQeMsgObject

		// The message type :
		int msgtype = msg.messageType;
		if (msgtype == MQC.MQMT_DATAGRAM) result.putInt(MQe.Msg_Style,MQe.Msg_Style_Datagram);
		else if (msgtype == MQC.MQMT_REPLY) result.putInt(MQe.Msg_Style,MQe.Msg_Style_Reply);
		else if (msgtype == MQC.MQMT_REQUEST) result.putInt(MQe.Msg_Style,MQe.Msg_Style_Request);
		
		// The priority :
		int priority = msg.priority;
		result.putByte(MQe.Msg_Priority,(byte)priority);

		// The replytoQueueName :
		String replytoQ = msg.replyToQueueName.trim();
		if (!replytoQ.equals(""))
		{
			result.putAscii(MQe.Msg_ReplyToQ,replytoQ);
		}

		//The replytoQueueManagerName :
		String replytoQMgr = msg.replyToQueueManagerName.trim();
		if (!replytoQMgr.equals(""))
		{
			result.putAscii(MQe.Msg_ReplyToQMgr,replytoQMgr);
		}
		
		// The message Id:
		byte[] msgID = msg.messageId;
		result.putArrayOfByte(MQe.Msg_MsgID,msgID);
		
		// The message correlation Id :
		byte[] msgcorrelID = msg.correlationId;
		String msgCorrelID2String = new String(msgcorrelID).trim(); 
		if (!msgCorrelID2String.equals(""))
		{
			result.putArrayOfByte(MQe.Msg_CorrelID,msgcorrelID);
		}
		
		// The putDateTime :
		Calendar MQTime = msg.putDateTime;
		long MQeTime = MQTime.getTime().getTime();
		result.putLong(MQe.Msg_Time,MQeTime);
		
		// The expiry Time :
		int expiry = msg.expiry;
		if (expiry > 0)
		{
			long time = System.currentTimeMillis( );  // get  the current time  
			long MQeExpiry = 100*(long)expiry + time;
			result.putLong(MQe.Msg_ExpireTime,MQeExpiry);
		}

		// The message Origin Queue Manager
		result.putOriginQMgr("MQeXMLBridge");
		
		return result;
	}
	/***************************************************************************
	 * Convert a MQe format message into a MQ format message. This method is
	 * called by the MQ/MQe bridge.
	 *
	 * @param msg     The MQe message object we are to transform.
	 * @param queue   A reference to the MQe queue the put the message.
	 *                (e.g. For determining default expiry time etc...)
	 * @param options A reference to a MQPutMessageOptions object in which any
	 *                required options for the put should be placed.
	 * @return        The equivalent MQ format message.
	 * @exception java.lang.Exception If the parameters are not valid, or if
	 * the message is in a format that this transformer does not understand.
	 ***************************************************************************/
public MQMessage transform(MQeMsgObject MQeMsg, MQeMQBridgeQueue queue, MQPutMessageOptions options) throws Exception
{
	MQMessage result;

	// We want to set all the options so :
	options.options = MQC.MQPMO_SET_ALL_CONTEXT;
	
	// First we get the class of the object passed, 
	// We put that class as an attribute of the root
	String ClassName = MQeMsg.getClass().getName();
	
	//We copy the MQeFields passed as parameter, since we don't want to modify it
	byte[] dump = MQeMsg.dump();
	MQeFields msg = (MQeFields) MQe.loadObject(ClassName);
	msg.restore( dump );
	
	/**
	 * This is a MQeMessage we have to transform to XML,	
	 * thus we have to transform all the MQeFields which are MQe specific, delete them
	 * and transform only the core data.
	 */
	result = new MQMessage();
	
	// set the message format to MQSTR, it is then human-readable in MQSeries
	result.format = MQC.MQFMT_STRING;
	
	// set the application name to MQe
	result.putApplicationName = "MQe";

	// set the putApplicationType to java
	result.putApplicationType = MQC.MQAT_JAVA;
	
	// add the priority if it's been set explicitly
	if (msg.contains(MQe.Msg_Priority))
	{
		result.priority = (int)msg.getByte(MQe.Msg_Priority);
		msg.delete(MQe.Msg_Priority);
	}

	// set the MQe.Msg_Style to the coresponding messageType
	if (msg.contains(MQe.Msg_Style))
	{
		int style = msg.getInt(MQe.Msg_Style);
		switch(style)
		{
			case MQe.Msg_Style_Datagram : result.messageType = MQC.MQMT_DATAGRAM;	break;
			case MQe.Msg_Style_Reply	: result.messageType = MQC.MQMT_REPLY;		break;
			case MQe.Msg_Style_Request	: result.messageType = MQC.MQMT_REQUEST;	break;
			default						: break;
		}
		msg.delete(MQe.Msg_Style);
	}

	// set the reply to queue
	if (msg.contains(MQe.Msg_ReplyToQ))
	{
		result.replyToQueueName = msg.getAscii(MQe.Msg_ReplyToQ);
		msg.delete(MQe.Msg_ReplyToQ);
	}

	// set the reply to queue manager name
	if (msg.contains(MQe.Msg_ReplyToQMgr))
	{
		result.replyToQueueManagerName = msg.getAscii(MQe.Msg_ReplyToQMgr);
		msg.delete(MQe.Msg_ReplyToQMgr);
	}
	
	// add the expiry time if it's been set explicitly
	// otherwise allow the default (no expiry time) to be set.
	// Note: Don't worry about values being set which are <1,
	// as if they are <1, the MQe-MQ bridge will suppress their
	// transmission to the MQ queue manager, as they have (by
	// definition) expired.
	// (Exception: If the value is MQC.MQEI_UNLIMITED then
	// no such checking/suppression is done).
	if (msg.contains(MQe.Msg_ExpireTime))
	{
		long    time   = System.currentTimeMillis( );       /* get current time     */
		char expiryFieldType = msg.dataType( MQe.Msg_ExpireTime );
		switch( expiryFieldType ) 
		{
			case MQeField.TypeLong :
				// The expiry time is a long, so it contains an absolute expiry
				// time... in milliseconds.
				// We need a delta time from the current time, as the expiry time
				// field of an MQ message contains the "time left before the message
				// expires" , in 1/10th of a second units.
				result.expiry = (int) ((msg.getLong(MQe.Msg_ExpireTime) - time)/100);
			break;

			case MQeField.TypeInt :
				// The expiry time is an int, so it is a delta expiry time
				// on top of the message creation time.
				// We need a delta time from the current time, as the expiry time
				// field of an MQ message contains the "time left before the message
				// expires" , in 1/10th of a second units.
				int deltaMilliSecs = msg.getInt(MQe.Msg_ExpireTime);
				long creationMilliSecs = msg.getLong( MQe.Msg_Time );
				long expiryMilliSecs = ((long)deltaMilliSecs) + creationMilliSecs - time ;
				result.expiry = (int) (expiryMilliSecs / 100) ;
			break;

			default:
				throw new Exception("ExpireTime neither int nor long!");
		} // End of the switch...
		msg.delete(MQe.Msg_ExpireTime);
	}

	//get the message time convert it to a Gregorian Calendar and set the latter in the MQMessage
	if(msg.contains(MQe.Msg_Time))
	{
		long MQeTime = msg.getLong(MQe.Msg_Time);
		Date date = new Date(MQeTime);
		GregorianCalendar MQTime = new GregorianCalendar();
		MQTime.setTime(date);
		result.putDateTime = MQTime;
		msg.delete(MQe.Msg_Time);
	}

	// convert the MQe MsgId to an MQ message Id if it is possible
	if(msg.contains(MQe.Msg_MsgID))
	{
		byte[] MsgId = msg.getArrayOfByte(MQe.Msg_MsgID);
		result.messageId = MsgId;
		msg.delete(MQe.Msg_MsgID);
	}

	//convert the message correl id
	if(msg.contains(MQe.Msg_CorrelID))
	{
		byte[] MsgCorrelId = msg.getArrayOfByte(MQe.Msg_CorrelID);
		result.correlationId = MsgCorrelId;
		msg.delete(MQe.Msg_CorrelID);
	}

	//before converting the rest of the data we must ensure that the MQeIndexEntryConstants.INDEX_TIME_ADDED
	//was deleted (bug in MQe which soo will be solve).
	if(msg.contains(MQeXMLConverter.INDEX_TIME_ADDED_v1))
	{
		msg.delete(MQeXMLConverter.INDEX_TIME_ADDED_v1);
	}
		
	// dump the remaining data to the MQ Series data
	result.write(converter.toXMLbyteArray(msg));
	return result;
	}
}
